==> objectify-alpha_27.2.tgz: objectify-alpha_27.2/time_stamp.c <==
/*
--             This file is part of the New World OS project
--                 Copyright (C) 2004-2008  QRW Software
--           J. Scott Edwards - j.scott.edwards.nwos@gmail.com 
--                      http://www.qrwsoftware.com
--                      http://nwos.sourceforge.com
--
--   This program is free software: you can redistribute it and/or modify
--   it under the terms of the GNU General Public License as published by
--   the Free Software Foundation, either version 3 of the License, or
--   (at your option) any later version.
--
--   This program is distributed in the hope that it will be useful,
--   but WITHOUT ANY WARRANTY; without even the implied warranty of
--   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
--   GNU General Public License for more details.
--
--   You should have received a copy of the GNU General Public License
--   along with this program, in the file LICENSE.  If not, see 
--   <http://www.gnu.org/licenses/>.
--
--   You can also contact me via paper mail at:
--
--      QRW Software
--      P.O. Box 27511
--      Salt Lake City, UT 84127-0511, USA.
--
--
-- $Log: time_stamp.c,v $
-- Revision 1.8  2008/04/13 23:24:26  jsedwards
-- Added conditional code to use gettimeofday instead of clock_gettime because
-- some OSes (like Mac OS X) don't have clock_gettime.
--
-- Revision 1.7  2007/07/01 19:44:12  jsedwards
-- Upgrade to GPLv3.
--
-- Revision 1.6  2006/12/01 14:37:23  jsedwards
-- Fix the year in the copyright.
--
-- Revision 1.5  2006/11/11 12:01:07  jsedwards
-- Update e-mail address to something that works.
--
-- Revision 1.4  2006/10/26 01:51:29  jsedwards
-- Merged alpha_05_branch back into main trunk.
--
-- Revision 1.3.2.6  2006/09/14 11:52:26  jsedwards
-- Changed to pass broken down time variables directly to inlined functions
-- instead of using intermediate variables.
--
-- Revision 1.3.2.5  2006/09/13 13:34:02  jsedwards
-- Changed to use new "insert" functions in time_stamp.h.
--
-- Revision 1.3.2.4  2006/09/10 20:42:13  jsedwards
-- Changed to use the new static inline routines in time_stamp.h to extract
-- the individual values instead of coding it inline.
--
-- Revision 1.3.2.3  2006/08/25 02:21:43  jsedwards
-- Changed to call timegm instead of mktime to convert broken down time to
-- a time_t value.  This way the time zone doesn't get stirred into the
-- calculation and mess things up.
--
-- Revision 1.3.2.2  2006/08/24 11:56:38  jsedwards
-- Added convert_time_struct_to_timeval routine.
--
-- Revision 1.3.2.1  2006/08/23 13:22:26  jsedwards
-- Added nwos_convert_timespec_to_time_stamp function.
--
-- Revision 1.3  2005/12/10 15:03:36  jsedwards
-- Fixed header to say the GPL is in the LICENSE file instead of COPYING.
--
-- Revision 1.2  2005/11/26 15:17:12  jsedwards
-- Made broken down time routines static (private), added 'nwos' prefix to
-- get time stamp function, and changed parameter to TimeStamp instead of
-- uint8[8].
--
-- Revision 1.1.1.1  2005/11/25 12:44:27  jsedwards
-- Copied from 'lab'.
--
-- Revision 1.3  2005/11/23 20:22:02  jsedwards
-- Changed to store as 8 bytes instead of a 64 bit integer.  This has the
-- advantage of working the same for both big and little endian machines.
--
-- Revision 1.2  2004/10/31 18:22:53  jsedwards
-- Changed to include 'time_stamp.h' instead of 'types.h' because the
--   time stamp stuff was moved out of 'types.h'.
--
-- Revision 1.1  2004/10/30 20:24:41  jsedwards
-- Initial Version.
--
--
*/

#include <stdio.h>
#include <time.h>
#include <stdlib.h>
#include "time_stamp.h"

  /*
  -- A time stamp is stored as 8 8-bit unsigned values,
  -- that represent broken down time.  See the header
  -- file for the break down of bits.
  */

static void unix_broken_down_time_to_timestamp(struct tm* bdt, uint32 usec, uint8 stamp[8])
{
  nwos_insert_year_into_time_stamp(bdt->tm_year + 1900, stamp);
  nwos_insert_month_into_time_stamp(bdt->tm_mon + 1, stamp);
  nwos_insert_day_of_month_into_time_stamp(bdt->tm_mday, stamp);
  nwos_insert_day_of_week_into_time_stamp(bdt->tm_wday + 1, stamp);
  nwos_insert_hour_into_time_stamp(bdt->tm_hour, stamp);
  nwos_insert_minute_into_time_stamp(bdt->tm_min, stamp);
  nwos_insert_second_into_time_stamp(bdt->tm_sec, stamp);
  nwos_insert_microsecond_into_time_stamp(usec, stamp);
}


static void timestamp_to_unix_broken_down_time(uint8 stamp[8], struct tm* bdt, uint32* usec)
{
  /* insert an error to test test */
  /* if (day_of_month == 13 && year == 1999 && month == 7) second++; */

  *usec = nwos_extract_microsecond_from_time_stamp(stamp);

  bdt->tm_sec  = nwos_extract_second_from_time_stamp(stamp);
  bdt->tm_min  = nwos_extract_minute_from_time_stamp(stamp);
  bdt->tm_hour = nwos_extract_hour_from_time_stamp(stamp);
  bdt->tm_mday = nwos_extract_day_of_month_from_time_stamp(stamp);
  bdt->tm_mon  = nwos_extract_month_from_time_stamp(stamp) - 1;
  bdt->tm_year = nwos_extract_year_from_time_stamp(stamp) - 1900;
  bdt->tm_wday = nwos_extract_day_of_week_from_time_stamp(stamp) - 1;
}


void nwos_get_time_stamp(TimeStamp stamp)
{
  struct timespec ts;
#ifdef HAVE_CLOCK_GETTIME
  if (clock_gettime(CLOCK_REALTIME, &ts) != 0) {
    perror("reading real-time clock");
    exit(1);
  }
#else
  struct timeval tv;
  if (gettimeofday(&tv, NULL) != 0) {
    perror("reading real-time clock");
    exit(1);
  }
  ts.tv_sec = tv.tv_sec;
  ts.tv_nsec = tv.tv_usec * 1000;
#endif
  nwos_convert_timespec_to_time_stamp(&ts, stamp);
}


void nwos_convert_timespec_to_time_stamp(struct timespec* ts, TimeStamp stamp)
{
  uint32 usec;
  struct tm bdt;

  if (ts->tv_nsec < 0L || ts->tv_nsec > 1000000000L) {
    fprintf(stderr, "nanoseconds in timespec out of range: %ld\n",
	    ts->tv_nsec);
    exit(1);
  }

  usec = ts->tv_nsec / 1000;   /* convert to microseconds */

  if (gmtime_r(&ts->tv_sec, &bdt) == NULL) {
    perror("gmtime_r");
    exit(1);
  }

  unix_broken_down_time_to_timestamp(&bdt, usec, stamp);
}


void nwos_convert_time_stamp_to_timeval(TimeStamp time_stamp, struct timeval* tv)
{
  uint32 usec;
  struct tm bdt;

  timestamp_to_unix_broken_down_time(time_stamp, &bdt, &usec);

  tv->tv_sec = timegm(&bdt);  /* as an alternative change the tz environment variable to UTC and then call mktime, then set it back */
  tv->tv_usec = usec;
}



/* Standalone test function - compile with -DTEST_ITER=n */

#ifdef TEST_ITER

#include <stdio.h>

void display_error(struct tm* bdt1, struct tm* bdt2, uint32 usec1, uint32 usec2, const char* name)
{
  printf("error in %s\n", name);
  printf("usec:    %lu %lu\n", usec1, usec2);
  printf("tm_sec:  %d %d\n", bdt1->tm_sec,  bdt2->tm_sec);
  printf("tm_min:  %d %d\n", bdt1->tm_min,  bdt2->tm_min);
  printf("tm_hour: %d %d\n", bdt1->tm_hour, bdt2->tm_hour);
  printf("tm_wday: %d %d\n", bdt1->tm_wday, bdt2->tm_wday);
  printf("tm_mday: %d %d\n", bdt1->tm_mday, bdt2->tm_mday);
  printf("tm_mon:  %d %d\n", bdt1->tm_mon,  bdt2->tm_mon);
  printf("tm_year: %d %d\n", bdt1->tm_year, bdt2->tm_year);
}

int main()
{
    int i;
    uint8 stamp[8];
    struct tm bdt1;
    struct tm bdt2;
    uint32 usec1;
    uint32 usec2;
    time_t sec;

    for (i = 0; i < TEST_ITER; i++)
    {
        /* generate a random time value */

	usec1 = rand() % 1000000;

	sec = rand() & 0x7fffffff;  /* always positive */

	if (gmtime_r(&sec, &bdt1) == NULL) {
	  perror("gmtime_r");
	  exit(1);
	}

	unix_broken_down_time_to_timestamp(&bdt1, usec1, stamp);
#if 0
	printf("%02x%02x%02x %02x%02x%02x%02x%02x\n",
	       stamp[0], stamp[1], stamp[2], stamp[3],
	       stamp[4], stamp[5], stamp[6], stamp[7]);
#endif
	timestamp_to_unix_broken_down_time(stamp, &bdt2, &usec2);

	if (usec1 != usec2) {
	  display_error(&bdt1, &bdt2, usec1, usec2, "usec");
	  exit(1);
	}
	if (bdt1.tm_sec != bdt2.tm_sec) {
	  display_error(&bdt1, &bdt2, usec1, usec2, "sec");
	  exit(1);
	}
	if (bdt1.tm_min != bdt2.tm_min) {
	  display_error(&bdt1, &bdt2, usec1, usec2, "min");
	  exit(1);
	}
	if (bdt1.tm_hour != bdt2.tm_hour) {
	  display_error(&bdt1, &bdt2, usec1, usec2, "hour");
	  exit(1);
	}
	if (bdt1.tm_wday != bdt2.tm_wday) {
	  display_error(&bdt1, &bdt2, usec1, usec2, "wday");
	  exit(1);
	}
	if (bdt1.tm_mday != bdt2.tm_mday) {
	  display_error(&bdt1, &bdt2, usec1, usec2, "mday");
	  exit(1);
	}
	if (bdt1.tm_mon != bdt2.tm_mon) {
	  display_error(&bdt1, &bdt2, usec1, usec2, "mon");
	  exit(1);
	}
	if (bdt1.tm_year != bdt2.tm_year) {
	  display_error(&bdt1, &bdt2, usec1, usec2, "year");
	  exit(1);
	}
    }

    return 0;
}

#endif

==> objectify-alpha_29.3.tgz: objectify-alpha_29.3/time_stamp.c <==
/*
--             This file is part of the New World OS project
--                 Copyright (C) 2004-2008  QRW Software
--           J. Scott Edwards - j.scott.edwards.nwos@gmail.com 
--                      http://www.qrwsoftware.com
--                      http://nwos.sourceforge.com
--
--   This program is free software: you can redistribute it and/or modify
--   it under the terms of the GNU General Public License as published by
--   the Free Software Foundation, either version 3 of the License, or
--   (at your option) any later version.
--
--   This program is distributed in the hope that it will be useful,
--   but WITHOUT ANY WARRANTY; without even the implied warranty of
--   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
--   GNU General Public License for more details.
--
--   You should have received a copy of the GNU General Public License
--   along with this program, in the file LICENSE.  If not, see 
--   <http://www.gnu.org/licenses/>.
--
--   You can also contact me via paper mail at:
--
--      QRW Software
--      P.O. Box 27511
--      Salt Lake City, UT 84127-0511, USA.
--
--
-- $Log: time_stamp.c,v $
-- Revision 1.9  2008/05/21 14:20:41  jsedwards
-- Added nwos_time_stamp_to_string function.
--
-- Revision 1.8  2008/04/13 23:24:26  jsedwards
-- Added conditional code to use gettimeofday instead of clock_gettime because
-- some OSes (like Mac OS X) don't have clock_gettime.
--
-- Revision 1.7  2007/07/01 19:44:12  jsedwards
-- Upgrade to GPLv3.
--
-- Revision 1.6  2006/12/01 14:37:23  jsedwards
-- Fix the year in the copyright.
--
-- Revision 1.5  2006/11/11 12:01:07  jsedwards
-- Update e-mail address to something that works.
--
-- Revision 1.4  2006/10/26 01:51:29  jsedwards
-- Merged alpha_05_branch back into main trunk.
--
-- Revision 1.3.2.6  2006/09/14 11:52:26  jsedwards
-- Changed to pass broken down time variables directly to inlined functions
-- instead of using intermediate variables.
--
-- Revision 1.3.2.5  2006/09/13 13:34:02  jsedwards
-- Changed to use new "insert" functions in time_stamp.h.
--
-- Revision 1.3.2.4  2006/09/10 20:42:13  jsedwards
-- Changed to use the new static inline routines in time_stamp.h to extract
-- the individual values instead of coding it inline.
--
-- Revision 1.3.2.3  2006/08/25 02:21:43  jsedwards
-- Changed to call timegm instead of mktime to convert broken down time to
-- a time_t value.  This way the time zone doesn't get stirred into the
-- calculation and mess things up.
--
-- Revision 1.3.2.2  2006/08/24 11:56:38  jsedwards
-- Added convert_time_struct_to_timeval routine.
--
-- Revision 1.3.2.1  2006/08/23 13:22:26  jsedwards
-- Added nwos_convert_timespec_to_time_stamp function.
--
-- Revision 1.3  2005/12/10 15:03:36  jsedwards
-- Fixed header to say the GPL is in the LICENSE file instead of COPYING.
--
-- Revision 1.2  2005/11/26 15:17:12  jsedwards
-- Made broken down time routines static (private), added 'nwos' prefix to
-- get time stamp function, and changed parameter to TimeStamp instead of
-- uint8[8].
--
-- Revision 1.1.1.1  2005/11/25 12:44:27  jsedwards
-- Copied from 'lab'.
--
-- Revision 1.3  2005/11/23 20:22:02  jsedwards
-- Changed to store as 8 bytes instead of a 64 bit integer.  This has the
-- advantage of working the same for both big and little endian machines.
--
-- Revision 1.2  2004/10/31 18:22:53  jsedwards
-- Changed to include 'time_stamp.h' instead of 'types.h' because the
--   time stamp stuff was moved out of 'types.h'.
--
-- Revision 1.1  2004/10/30 20:24:41  jsedwards
-- Initial Version.
--
--
*/

#include <stdio.h>
#include <time.h>
#include <stdlib.h>
#include "time_stamp.h"

  /*
  -- A time stamp is stored as 8 8-bit unsigned values,
  -- that represent broken down time.  See the header
  -- file for the break down of bits.
  */

static void unix_broken_down_time_to_timestamp(struct tm* bdt, uint32 usec, uint8 stamp[8])
{
  nwos_insert_year_into_time_stamp(bdt->tm_year + 1900, stamp);
  nwos_insert_month_into_time_stamp(bdt->tm_mon + 1, stamp);
  nwos_insert_day_of_month_into_time_stamp(bdt->tm_mday, stamp);
  nwos_insert_day_of_week_into_time_stamp(bdt->tm_wday + 1, stamp);
  nwos_insert_hour_into_time_stamp(bdt->tm_hour, stamp);
  nwos_insert_minute_into_time_stamp(bdt->tm_min, stamp);
  nwos_insert_second_into_time_stamp(bdt->tm_sec, stamp);
  nwos_insert_microsecond_into_time_stamp(usec, stamp);
}


static void timestamp_to_unix_broken_down_time(uint8 stamp[8], struct tm* bdt, uint32* usec)
{
  /* insert an error to test test */
  /* if (day_of_month == 13 && year == 1999 && month == 7) second++; */

  *usec = nwos_extract_microsecond_from_time_stamp(stamp);

  bdt->tm_sec  = nwos_extract_second_from_time_stamp(stamp);
  bdt->tm_min  = nwos_extract_minute_from_time_stamp(stamp);
  bdt->tm_hour = nwos_extract_hour_from_time_stamp(stamp);
  bdt->tm_mday = nwos_extract_day_of_month_from_time_stamp(stamp);
  bdt->tm_mon  = nwos_extract_month_from_time_stamp(stamp) - 1;
  bdt->tm_year = nwos_extract_year_from_time_stamp(stamp) - 1900;
  bdt->tm_wday = nwos_extract_day_of_week_from_time_stamp(stamp) - 1;
}


void nwos_get_time_stamp(TimeStamp stamp)
{
  struct timespec ts;
#ifdef HAVE_CLOCK_GETTIME
  if (clock_gettime(CLOCK_REALTIME, &ts) != 0) {
    perror("reading real-time clock");
    exit(1);
  }
#else
  struct timeval tv;
  if (gettimeofday(&tv, NULL) != 0) {
    perror("reading real-time clock");
    exit(1);
  }
  ts.tv_sec = tv.tv_sec;
  ts.tv_nsec = tv.tv_usec * 1000;
#endif
  nwos_convert_timespec_to_time_stamp(&ts, stamp);
}


void nwos_convert_timespec_to_time_stamp(struct timespec* ts, TimeStamp stamp)
{
  uint32 usec;
  struct tm bdt;

  if (ts->tv_nsec < 0L || ts->tv_nsec > 1000000000L) {
    fprintf(stderr, "nanoseconds in timespec out of range: %ld\n",
	    ts->tv_nsec);
    exit(1);
  }

  usec = ts->tv_nsec / 1000;   /* convert to microseconds */

  if (gmtime_r(&ts->tv_sec, &bdt) == NULL) {
    perror("gmtime_r");
    exit(1);
  }

  unix_broken_down_time_to_timestamp(&bdt, usec, stamp);
}


void nwos_convert_time_stamp_to_timeval(TimeStamp time_stamp, struct timeval* tv)
{
  uint32 usec;
  struct tm bdt;

  timestamp_to_unix_broken_down_time(time_stamp, &bdt, &usec);

  tv->tv_sec = timegm(&bdt);  /* as an alternative change the tz environment variable to UTC and then call mktime, then set it back */
  tv->tv_usec = usec;
}


const char* nwos_time_stamp_to_string(TimeStamp ts)
{
    static char result[20];

    snprintf(result, sizeof(result), "%u-%02u-%02u %02u:%02u:%02u",
	       nwos_extract_year_from_time_stamp(ts),
	       nwos_extract_month_from_time_stamp(ts),
	       nwos_extract_day_of_month_from_time_stamp(ts),
	       nwos_extract_hour_from_time_stamp(ts),
	       nwos_extract_minute_from_time_stamp(ts),
	       nwos_extract_second_from_time_stamp(ts));

    return result;
}


/* Standalone test function - compile with -DTEST_ITER=n */

#ifdef TEST_ITER

#include <stdio.h>

void display_error(struct tm* bdt1, struct tm* bdt2, uint32 usec1, uint32 usec2, const char* name)
{
  printf("error in %s\n", name);
  printf("usec:    %lu %lu\n", usec1, usec2);
  printf("tm_sec:  %d %d\n", bdt1->tm_sec,  bdt2->tm_sec);
  printf("tm_min:  %d %d\n", bdt1->tm_min,  bdt2->tm_min);
  printf("tm_hour: %d %d\n", bdt1->tm_hour, bdt2->tm_hour);
  printf("tm_wday: %d %d\n", bdt1->tm_wday, bdt2->tm_wday);
  printf("tm_mday: %d %d\n", bdt1->tm_mday, bdt2->tm_mday);
  printf("tm_mon:  %d %d\n", bdt1->tm_mon,  bdt2->tm_mon);
  printf("tm_year: %d %d\n", bdt1->tm_year, bdt2->tm_year);
}

int main()
{
    int i;
    uint8 stamp[8];
    struct tm bdt1;
    struct tm bdt2;
    uint32 usec1;
    uint32 usec2;
    time_t sec;

    for (i = 0; i < TEST_ITER; i++)
    {
        /* generate a random time value */

	usec1 = rand() % 1000000;

	sec = rand() & 0x7fffffff;  /* always positive */

	if (gmtime_r(&sec, &bdt1) == NULL) {
	  perror("gmtime_r");
	  exit(1);
	}

	unix_broken_down_time_to_timestamp(&bdt1, usec1, stamp);
#if 0
	printf("%02x%02x%02x %02x%02x%02x%02x%02x\n",
	       stamp[0], stamp[1], stamp[2], stamp[3],
	       stamp[4], stamp[5], stamp[6], stamp[7]);
#endif
	timestamp_to_unix_broken_down_time(stamp, &bdt2, &usec2);

	if (usec1 != usec2) {
	  display_error(&bdt1, &bdt2, usec1, usec2, "usec");
	  exit(1);
	}
	if (bdt1.tm_sec != bdt2.tm_sec) {
	  display_error(&bdt1, &bdt2, usec1, usec2, "sec");
	  exit(1);
	}
	if (bdt1.tm_min != bdt2.tm_min) {
	  display_error(&bdt1, &bdt2, usec1, usec2, "min");
	  exit(1);
	}
	if (bdt1.tm_hour != bdt2.tm_hour) {
	  display_error(&bdt1, &bdt2, usec1, usec2, "hour");
	  exit(1);
	}
	if (bdt1.tm_wday != bdt2.tm_wday) {
	  display_error(&bdt1, &bdt2, usec1, usec2, "wday");
	  exit(1);
	}
	if (bdt1.tm_mday != bdt2.tm_mday) {
	  display_error(&bdt1, &bdt2, usec1, usec2, "mday");
	  exit(1);
	}
	if (bdt1.tm_mon != bdt2.tm_mon) {
	  display_error(&bdt1, &bdt2, usec1, usec2, "mon");
	  exit(1);
	}
	if (bdt1.tm_year != bdt2.tm_year) {
	  display_error(&bdt1, &bdt2, usec1, usec2, "year");
	  exit(1);
	}
    }

    return 0;
}

#endif
